#ifndef FE_BOARD_H
#define FE_BOARD_H 

#include "board.h"
#include "fe_piece.h"

// width==height and an even number (graphism depends on these hypothesis
#define FE_BLOCK_WIDTH  21
#define FE_BLOCK_HEIGHT 21

/** Class that implements the specific management for the "Foul Egg" game. */
class FEBoard : public Board
{
 public:
    FEBoard(bool graphical, GiftPool *, QWidget *parent, const char *name=0);

	void init(int seed);
	
 private:
	// standard methods
	bool afterGlue(bool doAll);
	int  afterRemove(bool doAll);
	void remove();
	bool needRemoving();
	void checkBlock(uint i, uint j, uint value, uint &nb, bool set);
	void treatGroup(uint i, uint j, uint value, uint &nb, bool set);

	GPieceInfo *createPieceInfo() const { return new FEPieceInfo; }
	GPiece *createPiece() const         { return new FEPiece; }
	
	// graphic methods
	uint blockWidth() const { return FE_BLOCK_WIDTH; }
	uint blockHeight() const { return FE_BLOCK_HEIGHT; }
	const QColor *colors() const { return FE_COLORS; }
	uint drawCode(uint i, uint j) const;
	void lightRemoved(bool light);
	void drawBlock(QPainter *, const QRect &, const QColor &,
				   bool remove) const;
	void specialDraw(bool force);
	void specialDrawBlock(uint i, uint j, QPainter *p, const QRect &,
						  const QColor &) const;
	void paintEvent(QPaintEvent *e);

	// AI methods
	bool think(const Board &, uint &rotations, int &dec);
	int thinkingPoints();
	Board *newThinkingBoard();
	
	// Multiplayers methods
	uint gift();
	void putGift(uint);
	
	bool fragmentation;
	Matrix<int>  groups;
	Matrix<uint> codes;
	QArray<uint> rem;
	uint nbPuyos, chained, giftRest;
};

#endif
