#ifndef GPIECE_H
#define GPIECE_H

#include <qcolor.h>

class GPieceInfo
{
 public:
	GPieceInfo() {}
	virtual ~GPieceInfo() {}

	virtual uint nbBlocks() const = 0;
	virtual uint nbTypes() const  = 0;
	virtual uint nbForms() const  = 0;
	
	virtual const int *x(uint form) const = 0;
	virtual const int *y(uint form) const = 0;
	virtual uint nbConfigurations(uint type) const = 0;

	uint maxWidth()  const;
	uint maxHeight() const;
};

class GPiece
{
 public:
	typedef struct { int x, y, value; } Block;

 public:
    GPiece(uint nbBlocks) : blocks(nbBlocks) {}
	virtual ~GPiece() {}
	GPiece &operator =(const GPiece &);

	uint nbBlocks() const { return blocks.size(); }
	virtual uint nbConfigurations() const = 0;

	int  value(uint i) const        { return blocks[i].value; }
	uint pos(uint i, uint p) const  { return p + blocks[i].x; }
	uint line(uint i, uint l) const { return l - blocks[i].y; }

    int  minX() const;
    int  maxX() const;
	int  width() const { return maxX() - minX() + 1; }
    int  minY() const;
    int  maxY() const;
	int  height() const { return maxY() - minY() + 1; }

    void setRandomSeed(int _seed) { seed = _seed; }
	virtual void generateNext(int type = -1) = 0;
    void rotate(bool left);

 protected:
    QArray<Block> blocks;
	uint          type;
	int           seed;
};

#endif
