#include "inter.h"

#include <qmessagebox.h>
#include <qtimer.h>
#include <klocale.h>

#include "field.h"
#include "dialogs.h"
#include "version.h"
#include "defines.h"

const MPGameInfo KSIRTET_GAME_INFO = {
	KSIRTET_NAME, KSIRTET_MULTIPLAYER_ID,
	4,     // max nb local games
	500,   // interval
	TRUE,  // IA allowed
	0, 0   // no setting slots
};

const MPGameInfo KFOULEGGS_GAME_INFO = {
	KFOULEGGS_NAME, KFOULEGGS_MULTIPLAYER_ID,
	4,
	500,
	TRUE,
	0, 0
};

Interface::Interface(KAccel *kacc, QWidget *parent, const char *name)
 : MPSimpleInterface((TLGame ? KSIRTET_GAME_INFO : KFOULEGGS_GAME_INFO),
					 i18n((TLGame ? KSIRTET_NAME : KFOULEGGS_NAME)),
					 kacc, parent, name)
{
	setAction(i18n("Move left"),     "Move left",     SLOT(pMoveLeft()));
	setAction(i18n("Move right"),    "Move right",    SLOT(pMoveRight()));
	setAction(i18n("Drop down"),     "Drop down",     SLOT(pDropDown()));
	setAction(i18n("One line down"), "One line down", SLOT(pOneLineDown()));
	setAction(i18n("Rotate left"),   "Rotate left",   SLOT(pRotateLeft()));
	setAction(i18n("Rotate right"),  "Rotate right",  SLOT(pRotateRight()));

	QArray<const char **> defaultKeys(1);
	const char *oneHumanKeys[6]
		= { "Left", "Right", "Down", "Shift", "Up", "Return" };
	defaultKeys[0] = oneHumanKeys;
	setKeysConfiguration(defaultKeys);

	defaultKeys.resize(2);
	const char *twoHumansKeys_2[6] = { "F", "G", "D", "Space", "E", "Alt" };
	defaultKeys[0] = twoHumansKeys_2;
	defaultKeys[1] = oneHumanKeys;
	setKeysConfiguration(defaultKeys);
}

MPBoard *Interface::newBoard(uint)
{
	Field *f = new Field(this);
	connect(this, SIGNAL(toggleShowNextSignal(bool)),
			f, SLOT(toggleShowNext(bool)));
	connect(this, SIGNAL(toggleShadowSignal(bool)),
			f, SLOT(toggleShadow(bool)));
	return f;
}

void Interface::_init(bool server)
{
	if (server) data.resize(nbPlayers());
}

void Interface::_readGameOverData(QDataStream &s)
{
	// client side
	s >> gameOverData;
}

void Interface::_firstInit()
{
	// #### TODO (?)
}

void Interface::_treatInit()
{
	// send random seed
	ServerInitData sid;
	sid.seed = seedFromTime();
	for (uint i=0; i<nbPlayers(); i++) {
		sid.prevName = playerName(prev(i));
		sid.nextName = playerName(next(i));
		sid.name     = playerName(i);
		writingStream(i) << sid;
	}
}

bool Interface::_readPlayData()
{
	bool end = FALSE;
	for (uint i=0; i<nbPlayers(); i++) {
		readingStream(i) >> data[i];
		if (data[i].end) end = TRUE;
	}
	return end;
}

void Interface::_sendPlayData()
{
	ServerPlayData sd;
	for(uint i=0; i<nbPlayers(); i++) {
		sd.prevHeight = data[prev(i)].height;
		sd.nextHeight = data[next(i)].height;
		sd.gift       = data[prev(i)].gift;
		writingStream(i) << sd;
	}
}

void Interface::_sendGameOverData(QDataStream &s)
{
	// server side
	gameOverData.resize(nbPlayers());
	for (uint i=0; i<nbPlayers(); i++)
		readingStream(i) >> gameOverData[i];
	s << gameOverData;
}

void Interface::_showGameOverData(bool server)
{
	if (server && nbPlayers()==1 )
		WHighScores(this, &gameOverData[0]);
	else {
		debug("nb players : %i", gameOverData.size());
		for (uint i=0; i<gameOverData.size(); i++)
			debug("  player %i : %i %i", i, gameOverData[i].level,
				  gameOverData[i].score);
		// #### TODO
	}
}

uint Interface::prev(uint i) const
{
	if ( i==0 ) return nbPlayers()-1;
	else return i-1;
}

uint Interface::next(uint i) const
{
	if ( i==(nbPlayers()-1) ) return 0;
	else return i+1;
}
