#ifndef PLINE_H
#define PLINE_H

#include <qframe.h>
#include <qscrollbar.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qlayout.h>

#include "types.h"

/** Internal class : display a "player line" in netmeeting. */
class MeetingLine : public QFrame
{
 Q_OBJECT
	
 public:
	MeetingLine(bool readerIsServer, bool isOwner,
			   QWidget *parent = 0, const char *name = 0);
	virtual ~MeetingLine() {}

	MeetingCheckBox::Type type() const { return tcb->type(); }
	void setType(MeetingCheckBox::Type type) { tcb->setValue(type); }
	virtual void setText(const QString &text) = 0;

	void setData(const ExtData &ed);
	void data(ExtData &ed) const;
	virtual QString text() const = 0;

 signals:
	void typeChanged(MeetingCheckBox::Type);
	
 private slots:
	void _typeChanged(int t)
		{ emit typeChanged((MeetingCheckBox::Type)t); };

 protected:
	QHBoxLayout *hbl;
	
 private:
	MeetingCheckBox       *tcb;
	QLabel                *lname, *labH, *labAI;
	QValueList<BoardData>  bds;
};

/** Internal class : editable @ref MeetingLine. */
class EditMeetingLine : public MeetingLine
{
 Q_OBJECT
 
 public:
	EditMeetingLine(bool readerIsServer,
				   QWidget *parent = 0, const char *name = 0);
	
	void setText(const QString &str) { qle->setText(str); }
	QString text() const { return qle->text(); }
	
 signals:
	void textChanged(const QString &);

 private slots:
	void _textChanged(const QString &text) { emit textChanged(text); }
		
 private:
	QLineEdit *qle;
};

/** Internal class : not editable @ref MeetingLine. */
class ShowMeetingLine : public MeetingLine
{
 Q_OBJECT	
	
 public:
	ShowMeetingLine(bool readerIsServer,
				   QWidget *parent = 0, const char *name = 0);
	
	void setText(const QString &str) { lab->setText(str); }
	QString text() const { return lab->text(); }
	
 signals:
	void textChanged(const QString &);

 private slots:
	void _textChanged(const QString &text) { emit textChanged(text); };
		
 private:
	QLabel *lab;
};

class PlayerLine : public QFrame
{
 Q_OBJECT	
	
 public:
	PlayerLine(PlayerCheckBox::Type type, const QString &txt,
			   bool humanSetting, bool AISetting,
			   bool canBeEmpty, bool acceptAI,
			   QWidget *parent = 0, const char *name = 0);

	PlayerCheckBox::Type type() const { return pcb->type(); }
	QString name() const { return edit->text(); }

 signals:
	void setHuman();
	void setAI();
	void typeChanged(int);
	
 private slots:
	void setSlot();
	void typeChangedSlot(int);
	
 private:
	PlayerCheckBox *pcb;
	QLineEdit      *edit;
	QPushButton    *setting;
	bool            hs, as;
};

/** Internal class : scrolable list of widgets. */
class GWidgetList : public QWidget
{
 Q_OBJECT
		
 public:
	GWidgetList(uint interval, QWidget *parent = 0, const char * name = 0);
	
	QSize sizeHint() const;
	QSizePolicy sizePolicy() const;
	
	void remove(uint i);
	uint size() const { return widgets.count(); }

 protected:
	/** The widget must be created with this widget as parent. */
	void append(QWidget *);
	QWidget *widget(uint i) { return widgets.at(i); }

 private:
	QList<QWidget> widgets;
	uint interval;
	
	uint maxWidth() const;
	uint totalHeight() const;
};

template <class Type>
class WidgetList : public GWidgetList
{
 public:
	WidgetList(uint interval, QWidget *parent=0, const char *name=0)
	: GWidgetList(interval, parent, name) {}
	
	void append(Type *w) { GWidgetList::append(w); }
	Type *widget(uint i) { return (Type *)GWidgetList::widget(i); }
};

#endif // PLINE_H
