#include "main.h"

#include <stdio.h>
#include <qdatetime.h>
#include <klocale.h>
#include <kglobal.h>
#include <kmenubar.h>
#include <kconfig.h>

#include "defines.h"
#include "inter.h"
#include "version.h"
#include "field.h"

MainWidget::MainWidget()
{
	installEventFilter(this);
	kacc = new KAccel(this);

	inter = new Interface(kacc, this);
	inter->installEventFilter(this);
	inter->singleHuman();
	setView(inter);

	// KAccel initialization
	kacc->insertStdItem(KAccel::New, i18n("New game"));
	kacc->insertItem(i18n("Pause game"), "Pause game", "CTRL+P");
	kacc->insertStdItem(KAccel::Help);
	kacc->insertItem(i18n("High scores"), "High scores", "CTRL+H");

	// Kaccel connections
	kacc->connectItem(KAccel::Quit, this, SLOT(quit()));
	kacc->connectItem(KAccel::Print, this, SLOT(print()));
	kacc->connectItem(KAccel::New, inter, SLOT(start()));
	kacc->connectItem("Pause game", inter, SLOT(pause()));
	kacc->connectItem("High scores", this, SLOT(showHighScores()));

	kacc->readSettings();

	// Menu
	popup = new QPopupMenu;
	toggleId = popup->insertItem("", this, SLOT(toggleMenu()));
	popup->insertSeparator();
	int id = popup->insertItem(i18n("&New game"), inter, SLOT(start()));
	kacc->changeMenuAccel(popup, id, KAccel::New);
	id = popup->insertItem(i18n("&Pause game"), inter, SLOT(pause()));
	kacc->changeMenuAccel(popup, id, "Pause game");
	popup->insertSeparator();
	id = popup->insertItem(i18n("&High scores"),
						   this, SLOT(showHighScores()));
	kacc->changeMenuAccel(popup, id, "High scores");
	popup->insertSeparator();
	id = popup->insertItem(i18n("&Quit"), this, SLOT(quit()));
	kacc->changeMenuAccel(popup, id, KAccel::Quit);

	QPopupMenu *multip = new QPopupMenu;
	multip->insertItem(i18n("Single Human"), inter, SLOT(singleHuman()));
	multip->insertItem(i18n("Human vs Human"), inter, SLOT(humanVsHuman()));
	multip->insertItem(i18n("Human vs Computer"),
					   inter, SLOT(humanVsComputer()));
	multip->insertSeparator();
	multip->insertItem(i18n("More ..."), inter, SLOT(dialog()));
	
	options = new QPopupMenu;
	shadowId = options->insertItem(i18n("Shadow"), this, SLOT(toggleShadow()));
	showNextId = options->insertItem(i18n("Show next"),
									 this, SLOT(toggleShowNext()));
	options->insertItem(i18n("&Keys"), inter, SLOT(configureKeys()));

	QDate date(YEAR, MONTH, DAY);
	QString sdate = KGlobal::locale()->formatDate(date);
	QString s = i18n("%1 %2 (%3)\n\nby %4")
		.arg((TLGame ? KSIRTET_NAME : KFOULEGGS_NAME))
		.arg(VERSION).arg(sdate).arg(AUTHOR);
 	QPopupMenu *help = helpMenu(s);
	
	menu = new KMenuBar(this);
	menu->enableMoving(TRUE);
	menu->enableFloating(TRUE);
	menu->insertItem(i18n("&File"), popup);
	menu->insertItem(i18n("&Multiplayer"), multip);
	menu->insertItem(i18n("&Options"), options);
	menu->insertSeparator();
	menu->insertItem(i18n("&Help"), help);
	setMenu(menu);

	_toggleMenu(TRUE);
	_toggleShowNext(TRUE);
	_toggleShadow(TRUE);
}

void MainWidget::_toggleMenu(bool first)
{
	KConfig *conf = kapp->config();
	conf->setGroup(OP_GRP);
	bool show = conf->readBoolEntry(OP_MENUBAR, TRUE);
	if ( !first ) show = !show;
	conf->writeEntry(OP_MENUBAR, show);
	if (show) menu->show(); else menu->hide();
	popup->changeItem((show ? i18n("Hide menu") : i18n("Show menu")), toggleId);
}

bool MainWidget::eventFilter(QObject *, QEvent *e)
{
	switch (e->type()) {
	 case QEvent::MouseButtonPress:
		if ( ((QMouseEvent *)e)->button()!=RightButton ) return FALSE;
		popup->popup(QCursor::pos());
		return TRUE;
	 default : return FALSE;
	}
}

void MainWidget::_toggleShowNext(bool first)
{
	KConfig *conf = kapp->config();
	conf->setGroup(OP_GRP);
	bool show = conf->readBoolEntry(OP_SHOW_NEXT, TRUE);
	if ( !first ) show = !show;
	conf->writeEntry(OP_SHOW_NEXT, show);
	inter->toggleShowNext(show);
	options->setItemChecked(showNextId, show);
}

void MainWidget::_toggleShadow(bool first)
{
	KConfig *conf = kapp->config();
	conf->setGroup(OP_GRP);
	bool show = conf->readBoolEntry(OP_SHADOW, TRUE);
	if ( !first ) show = !show;
	conf->writeEntry(OP_SHADOW, show);
	inter->toggleShadow(show);
	options->setItemChecked(shadowId, show);
}

void MainWidget::quit()
{
	kapp->quit();
}

//-----------------------------------------------------------------------------
void usage()
{
	printf("There is now two games in one executable.\n"
		   "Use one of the following options :\n"
           " --sirtet   : Tetris-like game\n"
		   " --fouleggs : Foul Eggs game (also known as PuyoPuyo)\n");
}

void checkOptions(int argc, char **argv)
{
	for (int i=0; i<argc; i++) {
		if ( strcmp(argv[i], "--sirtet")==0 ) {
			TLGame = TRUE;
			return;
		}
		if ( strcmp(argv[i], "--fouleggs")==0 ) {
			TLGame = FALSE;
			return;
		}
	}
	usage();
	exit(1);
}

int main(int argc, char **argv)
{
	checkOptions(argc, argv);
	KApplication a(argc, argv, (TLGame ? KSIRTET_NAME : KFOULEGGS_NAME));
	if ( !TLGame ) KGlobal::locale()->insertCatalogue("ksirtet");

	MainWidget *mw = new MainWidget;
	a.setMainWidget(mw);
	mw->show();
	return a.exec();
}
